@@ -17,14 +17,13 @@ from functools import wraps
 from os.path import exists, join
 
 import pytest
+from conftest import mkurl
 from fs.errors import BackReferenceError, DestinationExistsError, \
     DirectoryNotEmptyError, FSError, InvalidPathError, RemoteConnectionError, \
     ResourceError, ResourceInvalidError, ResourceNotFoundError, \
     UnsupportedError
 from mock import Mock
 from XRootD.client.responses import XRootDStatus
-
-from conftest import mkurl
 from xrootdpyfs import XRootDPyFile, XRootDPyFS
 from xrootdpyfs.utils import spliturl
 
@@ -313,7 +312,7 @@ def test_open(tmppath):
     """Test fs.open()"""
     # Create a file to open.
     file_name = 'data/testa.txt'
-    contents = 'testa.txt\n'
+    expected_content = b'testa.txt\n'
     xrd_rooturl = mkurl(tmppath)
 
     # Open file w/ xrootd
@@ -322,7 +321,7 @@ def test_open(tmppath):
     assert xfile
     assert xfile.path.endswith("data/testa.txt")
     assert type(xfile) == XRootDPyFile
-    assert xfile.read() == contents
+    assert xfile.read() == expected_content
     xfile.close()
 
     # Test passing of querystring.
@@ -331,7 +330,7 @@ def test_open(tmppath):
     assert xfile
     assert xfile.path.endswith("data/testa.txt?xrd.wantprot=krb5")
     assert type(xfile) == XRootDPyFile
-    assert xfile.read() == contents
+    assert xfile.read() == expected_content
     xfile.close()
 
 
@@ -528,19 +527,23 @@ def test_movedir_good(tmppath):
     assert fs.exists(dst_folder_exists)
     assert not fs.exists(dst_folder_new)
 
+    # Move to new folder (without trailing slash).
     fs.movedir(src_exists, dst_new)
     assert not fs.exists(src_exists) and fs.exists(dst_new)
 
-    fs.movedir(dst_new, src_exists)
+    fs.movedir(dst_new, src_exists)  # reset
+    # Move to new folder (with trailing slash).
     fs.movedir(src_exists, dst_folder_new)
     assert not fs.exists(src_exists) and fs.exists(dst_folder_new)
 
-    fs.movedir(dst_folder_new, src_exists)
+    fs.movedir(dst_folder_new, src_exists)  # reset
+    # Move to existing filer with overwrite (i.e. will remove destination)
     fs.movedir(src_exists, dst_exists, overwrite=True)
     assert not fs.exists(src_exists) and fs.exists(dst_exists)
     assert fs.isdir(dst_exists)
 
-    fs.movedir(dst_exists, src_exists)
+    fs.movedir(dst_exists, src_exists)  # reset
+    # Move to existing folder with overwrite (i.e. will remove destination)
     fs.movedir(src_exists, dst_folder_exists, overwrite=True)
     assert not fs.exists(src_exists) and fs.exists(dst_folder_exists)
     assert fs.isdir(dst_folder_exists)
@@ -578,15 +581,6 @@ def test_move_bad(tmppath):
     pytest.raises(ResourceNotFoundError, fs.move, src_new, dst_folder_exists)
     pytest.raises(ResourceNotFoundError, fs.move, src_new, dst_folder_new)
 
-    pytest.raises(
-        ResourceNotFoundError, fs.move, src_new, dst_exists)
-    pytest.raises(
-        ResourceNotFoundError, fs.move, src_new, dst_new)
-    pytest.raises(
-        ResourceNotFoundError, fs.move, src_new, dst_folder_exists)
-    pytest.raises(
-        ResourceNotFoundError, fs.move, src_new, dst_folder_new)
-
 
 def test_movedir_bad(tmppath):
     """Test move file."""
@@ -602,11 +596,9 @@ def test_movedir_bad(tmppath):
     dst_folder_new = "data/anothernewfolder/"
 
     # Destination exists
-    pytest.raises(
-        DestinationExistsError, fs.movedir, src_folder_exists, dst_exists)
     pytest.raises(
         DestinationExistsError, fs.movedir, src_folder_exists,
-        src_folder_exists)
+        dst_exists)
     pytest.raises(
         DestinationExistsError, fs.movedir, src_folder_exists,
         dst_folder_exists)
